@@ -6,8 +6,9 @@
 //
 // Copyright 2013, 2014 Igalia S.L.
 // Copyright 2014 Luigi Scarso <luigi.scarso@gmail.com>
-// Copyright 2014, 2018, 2019 Albert Astals Cid <aacid@kde.org>
+// Copyright 2014, 2018, 2019, 2021 Albert Astals Cid <aacid@kde.org>
 // Copyright 2018 Adam Reichold <adam.reichold@t-online.de>
+// Copyright 2021 Adrian Johnson <ajohnson@redneon.com>
 //
 //========================================================================
 
@@ -124,11 +125,11 @@ public:
     Owner getOwner() const { return owner; }
     const char *getTypeName() const;
     const char *getOwnerName() const;
-    Object *getValue() const { return &value; }
+    const Object *getValue() const { return &value; }
     static Object *getDefaultValue(Type type);
 
     // The caller gets the ownership of the return GooString and is responsible of deleting it
-    GooString *getName() const { return type == UserProperty ? name.copy() : new GooString(getTypeName()); }
+    std::unique_ptr<GooString> getName() const { return std::make_unique<GooString>(type == UserProperty ? name.c_str() : getTypeName()); }
 
     // The revision is optional, and defaults to zero.
     unsigned int getRevision() const { return revision; }
@@ -149,8 +150,8 @@ private:
     Type type;
     Owner owner;
     unsigned int revision;
-    mutable GooString name;
-    mutable Object value;
+    GooString name;
+    Object value;
     bool hidden;
     GooString *formatted;
 
@@ -253,14 +254,16 @@ public:
     // Optional ISO language name, e.g. en_US
     GooString *getLanguage()
     {
-        if (!isContent() && s->language)
+        if (!isContent() && s->language) {
             return s->language;
+        }
         return parent ? parent->getLanguage() : nullptr;
     }
     const GooString *getLanguage() const
     {
-        if (!isContent() && s->language)
+        if (!isContent() && s->language) {
             return s->language;
+        }
         return parent ? parent->getLanguage() : nullptr;
     }
 
@@ -268,8 +271,9 @@ public:
     unsigned int getRevision() const { return isContent() ? 0 : s->revision; }
     void setRevision(unsigned int revision)
     {
-        if (isContent())
+        if (isContent()) {
             s->revision = revision;
+        }
     }
 
     // Optional element title, in human-readable form.
@@ -327,9 +331,10 @@ public:
 
     const TextSpanArray getTextSpans() const
     {
-        if (!isContent())
+        if (!isContent()) {
             return TextSpanArray();
-        MarkedContentOutputDev mcdev(getMCID());
+        }
+        MarkedContentOutputDev mcdev(getMCID(), stmRef);
         return getTextSpansInternal(mcdev);
     }
 
@@ -370,8 +375,8 @@ private:
             Ref ref;
         };
 
-        ContentData(int mcidA) : mcid(mcidA) { }
-        ContentData(const Ref r) { ref = r; }
+        explicit ContentData(int mcidA) : mcid(mcidA) { }
+        explicit ContentData(const Ref r) { ref = r; }
     };
 
     // Common data
@@ -379,6 +384,7 @@ private:
     StructTreeRoot *treeRoot;
     StructElement *parent;
     mutable Object pageRef;
+    Object stmRef;
 
     union {
         StructData *s;
